/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Unzip.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.autostart.unzip;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.ow2.jonas.autostart.utility.Output;

/**
 * This class unzipes a zip file.
 * @author skabore
 */
public class Unzip {

    /**
     * File to unzip.
     */
    private File src;

    /**
     * Location of unzipped file.
     */
    private File dest;

    /**
     * ZipFile attribute.
     */
    private static ZipFile zf;

    /**
     * End Of File to unzip.
     */
    private static final int EOF = -1;

    /**
     * The output stream.
     */
    private Output output;

    /**
     * The verboseMode.
     */
    private boolean verboseMode;

    /**
     * Build a new Unzip class.
     */
    public Unzip() {
        verboseMode = false;
    }

    /**
     * Build a new Unzip class.
     * @param output the output stream.
     */
    public Unzip(final Output output) {
        this.output = output;
        verboseMode = false;
    }

    /**
     * @return source of file.
     */
    public File getSrc() {
        return src;
    }

    /**
     * This method modifies the source of file to unzip.
     * @param src the source of file
     */
    public void setSrc(final File src) {
        this.src = src;
    }

    /**
     * @return destination of file.
     */
    public File getDest() {
        return dest;
    }

    /**
     * This method modifies the destination of unzipped file.
     * @param dest the destination of file
     */
    public void setDest(final File dest) {
        this.dest = dest;
    }

    /**
     * @return the verboseMode.
     */
    public boolean getVerboseMode() {
        return verboseMode;
    }

    /**
     * Set the verboseMode.
     * @param verboseMode the verboseMode.
     */
    public void setVerboseMode(final boolean verboseMode) {
        this.verboseMode = verboseMode;
    }

    /**
     * This method unzips file.
     */
    public void execute() throws UnzipException {
        Enumeration<? extends ZipEntry> enumr;
        try {
            zf = new ZipFile(src);
            enumr = zf.entries();
            while (enumr.hasMoreElements()) {
                ZipEntry target = enumr.nextElement();
                if (verboseMode) {
                    output.write(target.getName() + " unpacked.");
                }
                saveEntry(target);
            }
        } catch (FileNotFoundException e) {
            output.write(e.getMessage());
            throw new UnzipException("Unable to unzip '" + src + "'", e);
        } catch (ZipException e) {
            output.write(e.getMessage());
            throw new UnzipException("Unable to unzip '" + src + "'", e);
        } catch (IOException e) {
            output.write(e.getMessage());
            throw new UnzipException("Unable to unzip '" + src + "'", e);
        } finally {
            try {
                if (zf != null) {
                    zf.close();
                }
            } catch (IOException ex) {
                output.write(ex.getMessage());
            }
        }
    }

    /**
     * Save entry in the target.
     * @param target the target
     * @throws IOException the IOException
     */
    public void saveEntry(final ZipEntry target) throws IOException {
        try {
            File file = new File(dest.getAbsolutePath(), target.getName());
            if (file.exists()) {
                file.delete();
            }
            if (target.isDirectory()) {
                file.mkdirs();
            } else {
                InputStream is = zf.getInputStream(target);
                BufferedInputStream bis = new BufferedInputStream(is);
                File dir = new File(file.getParent());
                dir.mkdirs();
                FileOutputStream fos = new FileOutputStream(file);
                BufferedOutputStream bos = new BufferedOutputStream(fos);
                int c;
                while ((c = bis.read()) != EOF) {
                    bos.write((byte) c);
                }
                bos.close();
                fos.close();
            }
        } catch (IOException e) {
            output.write(e.getMessage());
        }
    }
}
